
/*-------------------------------------------------------------*/
/* Name : netwib_filename_exists
   Description :
     Check if a regular file exists.
   Input parameter(s) :
     *pfilename : filename
   Input/output parameter(s) :
   Output parameter(s) :
     *pyes : if file exists
   Normal return values :
     NETWIB_ERR_OK : ok
*/
netwib_err netwib_filename_exists(netwib_constbuf *pfilename,
                                  netwib_bool *pyes);

/*-------------------------------------------------------------*/
/***************************************************************
 * Netwib does not support files which have a size > 0xFFFFFFFF*
 * (4Gbytes). Indeed, netwib is a network library, and such big*
 * size does not make sense. If a function returns the error   *
 * NETWIB_ERR_SPFILE4G, then you cannot use netwib to deal with*
 * it : just create your own function (dealing for example     *
 * with 64bits sizes).                                         *
 ***************************************************************/
/* Name : netwib_filename_size
   Description :
     Get size of a file.
   Input parameter(s) :
     *pfilename : filename
   Input/output parameter(s) :
   Output parameter(s) :
     *psize : size of the file
   Normal return values :
     NETWIB_ERR_OK : ok
     NETWIB_ERR_OKNOTFOUND : file was not found
*/
netwib_err netwib_filename_size(netwib_constbuf *pfilename,
                                netwib_uint32 *psize);

/*-------------------------------------------------------------*/
/* Name : netwib_filename_create
   Description :
     Create an empty file.
   Input parameter(s) :
     *pfilename : filename
   Input/output parameter(s) :
   Output parameter(s) :
   Normal return values :
     NETWIB_ERR_OK : ok
*/
netwib_err netwib_filename_create(netwib_constbuf *pfilename);

/*-------------------------------------------------------------*/
/* Name : netwib_filename_remove
   Description :
     Remove a file.
   Input parameter(s) :
     *pfilename : filename
   Input/output parameter(s) :
   Output parameter(s) :
   Normal return values :
     NETWIB_ERR_OK : ok
     NETWIB_ERR_OKNOTFOUND : file was not found
*/
netwib_err netwib_filename_remove(netwib_constbuf *pfilename);

/*-------------------------------------------------------------*/
/* Name : netwib_filename_rename
   Description :
     Rename a file.
   Input parameter(s) :
     *poldfilename : old filename
     *pnewfilename : new filename
   Input/output parameter(s) :
   Output parameter(s) :
   Normal return values :
     NETWIB_ERR_OK : ok
     NETWIB_ERR_OKNOTFOUND : file was not found
*/
netwib_err netwib_filename_rename(netwib_constbuf *poldfilename,
                                  netwib_constbuf *pnewfilename);

/*-------------------------------------------------------------*/
/* Name : netwib_filename_copy
   Description :
     Copy a file.
   Input parameter(s) :
     *poldfilename : old filename
     *pnewfilename : new filename
   Input/output parameter(s) :
   Output parameter(s) :
   Normal return values :
     NETWIB_ERR_OK : ok
     NETWIB_ERR_OKNOTFOUND : file was not found
*/
netwib_err netwib_filename_copy(netwib_constbuf *poldfilename,
                                netwib_constbuf *pnewfilename);



